@@ -10,15 +10,12 @@ from datetime import date
 import pandas as pd
 import numpy as np
 from perspective import Table, PerspectiveWidget
-from ..common import superstore
-
-DF = superstore(200)
 
 
 class TestWidgetPandas:
 
-    def test_widget_load_table_df(self):
-        table = Table(DF)
+    def test_widget_load_table_df(self, superstore):
+        table = Table(superstore)
         widget = PerspectiveWidget(table)
         assert widget.table.schema() == {'index': int, 'Country': str, 'Region': str, 'Category': str, 'City': str, 'Customer ID': str, 'Discount': float,
                                          'Order Date': date, 'Order ID': str, 'Postal Code': str, 'Product ID': str, 'Profit': float, 'Quantity': int,
@@ -28,30 +25,30 @@ class TestWidgetPandas:
                                                  'Product ID', 'Profit', 'Quantity', 'Region', 'Row ID', 'Sales', 'Segment', 'Ship Date',
                                                  'Ship Mode', 'State', 'Sub-Category'])
         view = widget.table.view()
-        assert view.num_rows() == len(DF)
-        assert view.num_columns() == len(DF.columns) + 1  # index
+        assert view.num_rows() == len(superstore)
+        assert view.num_columns() == len(superstore.columns) + 1  # index
 
-    def test_widget_load_data_df(self):
-        widget = PerspectiveWidget(DF)
+    def test_widget_load_data_df(self, superstore):
+        widget = PerspectiveWidget(superstore)
         assert sorted(widget.columns) == sorted(['index', 'Category', 'City', 'Country', 'Customer ID', 'Discount', 'Order Date', 'Order ID', 'Postal Code',
                                                  'Product ID', 'Profit', 'Quantity', 'Region', 'Row ID', 'Sales', 'Segment', 'Ship Date',
                                                  'Ship Mode', 'State', 'Sub-Category'])
         view = widget.table.view()
-        assert view.num_rows() == len(DF)
+        assert view.num_rows() == len(superstore)
         assert view.num_columns() == 20
 
-    def test_widget_load_series(self):
-        series = pd.Series(DF["Profit"].values, name="profit")
+    def test_widget_load_series(self, superstore):
+        series = pd.Series(superstore["Profit"].values, name="profit")
         widget = PerspectiveWidget(series)
         assert widget.table.schema() == {'index': int, 'profit': float}
 
         assert sorted(widget.columns) == sorted(["index", "profit"])
         view = widget.table.view()
-        assert view.num_rows() == len(DF)
+        assert view.num_rows() == len(superstore)
         assert view.num_columns() == 2
 
-    def test_widget_load_pivot_table(self):
-        pivot_table = pd.pivot_table(DF, values='Discount', index=['Country', 'Region'], columns=['Category', 'Segment'])
+    def test_widget_load_pivot_table(self, superstore):
+        pivot_table = pd.pivot_table(superstore, values='Discount', index=['Country', 'Region'], columns=['Category', 'Segment'])
         widget = PerspectiveWidget(pivot_table)
         assert widget.group_by == ['Country', 'Region']
         assert widget.split_by == ['Category', 'Segment']
@@ -61,8 +58,8 @@ class TestWidgetPandas:
         assert view.num_rows() == 60
         assert view.num_columns() == 6
 
-    def test_widget_load_pivot_table_with_user_pivots(self):
-        pivot_table = pd.pivot_table(DF, values='Discount', index=['Country', 'Region'], columns='Category')
+    def test_widget_load_pivot_table_with_user_pivots(self, superstore):
+        pivot_table = pd.pivot_table(superstore, values='Discount', index=['Country', 'Region'], columns='Category')
         widget = PerspectiveWidget(pivot_table, group_by=["Category", "Segment"])
         assert widget.group_by == ['Category', 'Segment']
         assert widget.split_by == []
@@ -72,33 +69,33 @@ class TestWidgetPandas:
         assert view.num_rows() == 5
         assert view.num_columns() == 6
 
-    def test_widget_load_group_by(self):
-        df_pivoted = DF.set_index(['Country', 'Region'])
+    def test_widget_load_group_by(self, superstore):
+        df_pivoted = superstore.set_index(['Country', 'Region'])
         widget = PerspectiveWidget(df_pivoted)
         assert widget.group_by == ['Country', 'Region']
         assert widget.split_by == []
         assert sorted(widget.columns) == sorted(['index', 'Category', 'Country', 'City', 'Customer ID', 'Discount', 'Order Date', 'Order ID', 'Postal Code',
                                                 'Product ID', 'Profit', 'Quantity', 'Region', 'Row ID', 'Sales', 'Segment', 'Ship Date',
                                                 'Ship Mode', 'State', 'Sub-Category'])
-        assert widget.table.size() == 200
+        assert widget.table.size() == 100
         view = widget.table.view()
-        assert view.num_rows() == len(DF)
-        assert view.num_columns() == len(DF.columns) + 1  # index
+        assert view.num_rows() == len(superstore)
+        assert view.num_columns() == len(superstore.columns) + 1  # index
 
-    def test_widget_load_group_by_with_user_pivots(self):
-        df_pivoted = DF.set_index(['Country', 'Region'])
+    def test_widget_load_group_by_with_user_pivots(self, superstore):
+        df_pivoted = superstore.set_index(['Country', 'Region'])
         widget = PerspectiveWidget(df_pivoted, group_by=["Category", "Segment"])
         assert widget.group_by == ['Category', 'Segment']
         assert widget.split_by == []
         assert sorted(widget.columns) == sorted(['index', 'Category', 'Country', 'City', 'Customer ID', 'Discount', 'Order Date', 'Order ID', 'Postal Code',
                                                 'Product ID', 'Profit', 'Quantity', 'Region', 'Row ID', 'Sales', 'Segment', 'Ship Date',
                                                 'Ship Mode', 'State', 'Sub-Category'])
-        assert widget.table.size() == 200
+        assert widget.table.size() == 100
         view = widget.table.view()
-        assert view.num_rows() == len(DF)
-        assert view.num_columns() == len(DF.columns) + 1  # index
+        assert view.num_rows() == len(superstore)
+        assert view.num_columns() == len(superstore.columns) + 1  # index
 
-    def test_widget_load_split_by(self):
+    def test_widget_load_split_by(self, superstore):
         arrays = [np.array(['bar', 'bar', 'bar', 'bar', 'baz', 'baz', 'baz', 'baz', 'foo', 'foo', 'foo', 'foo', 'qux', 'qux', 'qux', 'qux']),
                   np.array(['one', 'one', 'two', 'two', 'one', 'one', 'two', 'two', 'one', 'one', 'two', 'two', 'one', 'one', 'two', 'two']),
                   np.array(['X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y'])]
@@ -110,7 +107,7 @@ class TestWidgetPandas:
         assert widget.split_by == ['first', 'second', 'third']
         assert widget.group_by == ['index']
 
-    def test_widget_load_split_by_preserve_user_settings(self):
+    def test_widget_load_split_by_preserve_user_settings(self, superstore):
         arrays = [np.array(['bar', 'bar', 'bar', 'bar', 'baz', 'baz', 'baz', 'baz', 'foo', 'foo', 'foo', 'foo', 'qux', 'qux', 'qux', 'qux']),
                   np.array(['one', 'one', 'two', 'two', 'one', 'one', 'two', 'two', 'one', 'one', 'two', 'two', 'one', 'one', 'two', 'two']),
                   np.array(['X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y'])]
@@ -122,7 +119,7 @@ class TestWidgetPandas:
         assert widget.split_by == ['first', 'second', 'third']
         assert widget.group_by == ['index']
 
-    def test_pivottable_values_index(self):
+    def test_pivottable_values_index(self, superstore):
         arrays = {'A':['bar', 'bar', 'bar', 'bar', 'baz', 'baz', 'baz', 'baz', 'foo', 'foo', 'foo', 'foo', 'qux', 'qux', 'qux', 'qux'],
                 'B':['one', 'one', 'two', 'two', 'one', 'one', 'two', 'two', 'one', 'one', 'two', 'two', 'one', 'one', 'two', 'two'],
                 'C':['X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y', 'X', 'Y'],
@@ -135,8 +132,8 @@ class TestWidgetPandas:
         assert widget.split_by == ['B', 'C']
         assert widget.group_by == ['A']
 
-    def test_pivottable_multi_values(self):
-        pt = pd.pivot_table(DF, values = ['Discount','Sales'], index=['Country','Region'],aggfunc={'Discount':'count','Sales':'sum'},columns=["State","Quantity"])
+    def test_pivottable_multi_values(self, superstore):
+        pt = pd.pivot_table(superstore, values = ['Discount','Sales'], index=['Country','Region'],aggfunc={'Discount':'count','Sales':'sum'},columns=["State","Quantity"])
         widget = PerspectiveWidget(pt)
         assert widget.columns == ['Discount', 'Sales']
         assert widget.split_by == ['State', 'Quantity']
